@@ -1,13 +1,15 @@
 """Provide the 'autogenerate' feature which can produce migration operations
 automatically."""
 
-from ..operations import ops
-from . import render
-from . import compare
-from .. import util
-from sqlalchemy.engine.reflection import Inspector
 import contextlib
 
+from sqlalchemy import inspect
+
+from . import compare
+from . import render
+from .. import util
+from ..operations import ops
+
 
 def compare_metadata(context, metadata):
     """Compare a database schema to that given in a
@@ -29,22 +31,23 @@ def compare_metadata(context, metadata):
         from sqlalchemy.schema import SchemaItem
         from sqlalchemy.types import TypeEngine
         from sqlalchemy import (create_engine, MetaData, Column,
-                Integer, String, Table)
+                Integer, String, Table, text)
         import pprint
 
         engine = create_engine("sqlite://")
 
-        engine.execute('''
-            create table foo (
-                id integer not null primary key,
-                old_data varchar,
-                x integer
-            )''')
+        with engine.begin() as conn:
+            conn.execute(text('''
+                create table foo (
+                    id integer not null primary key,
+                    old_data varchar,
+                    x integer
+                )'''))
 
-        engine.execute('''
-            create table bar (
-                data varchar
-            )''')
+            conn.execute(text('''
+                create table bar (
+                    data varchar
+                )'''))
 
         metadata = MetaData()
         Table('foo', metadata,
@@ -112,8 +115,6 @@ def produce_migrations(context, metadata):
     :class:`.MigrationScript` object.   For an example of what this looks like,
     see the example in :ref:`customizing_revision`.
 
-    .. versionadded:: 0.8.0
-
     .. seealso::
 
         :func:`.compare_metadata` - returns more fundamental "diff"
@@ -136,11 +137,12 @@ def produce_migrations(context, metadata):
 
 def render_python_code(
     up_or_down_op,
-    sqlalchemy_module_prefix='sa.',
-    alembic_module_prefix='op.',
+    sqlalchemy_module_prefix="sa.",
+    alembic_module_prefix="op.",
     render_as_batch=False,
     imports=(),
     render_item=None,
+    migration_context=None,
 ):
     """Render Python code given an :class:`.UpgradeOps` or
     :class:`.DowngradeOps` object.
@@ -150,16 +152,25 @@ def render_python_code(
 
     """
     opts = {
-        'sqlalchemy_module_prefix': sqlalchemy_module_prefix,
-        'alembic_module_prefix': alembic_module_prefix,
-        'render_item': render_item,
-        'render_as_batch': render_as_batch,
+        "sqlalchemy_module_prefix": sqlalchemy_module_prefix,
+        "alembic_module_prefix": alembic_module_prefix,
+        "render_item": render_item,
+        "render_as_batch": render_as_batch,
     }
 
-    autogen_context = AutogenContext(None, opts=opts)
+    if migration_context is None:
+        from ..runtime.migration import MigrationContext
+        from sqlalchemy.engine.default import DefaultDialect
+
+        migration_context = MigrationContext.configure(
+            dialect=DefaultDialect()
+        )
+
+    autogen_context = AutogenContext(migration_context, opts=opts)
     autogen_context.imports = set(imports)
-    return render._indent(render._render_cmd_body(
-        up_or_down_op, autogen_context))
+    return render._indent(
+        render._render_cmd_body(up_or_down_op, autogen_context)
+    )
 
 
 def _render_migration_diffs(context, template_args):
@@ -208,7 +219,7 @@ class AutogenContext(object):
     connected to the database backend being compared.
 
     This is obtained from the :attr:`.MigrationContext.bind` and is
-    utimately set up in the ``env.py`` script.
+    ultimately set up in the ``env.py`` script.
 
     """
 
@@ -225,10 +236,8 @@ class AutogenContext(object):
 
     The directives are to be rendered into the ``${imports}`` section
     of a script template.  The set is normally empty and can be modified
-    within hooks such as the :paramref:`.EnvironmentContext.configure.render_item`
-    hook.
-
-    .. versionadded:: 0.8.3
+    within hooks such as the
+    :paramref:`.EnvironmentContext.configure.render_item` hook.
 
     .. seealso::
 
@@ -240,47 +249,51 @@ class AutogenContext(object):
     """The :class:`.MigrationContext` established by the ``env.py`` script."""
 
     def __init__(
-            self, migration_context, metadata=None,
-            opts=None, autogenerate=True):
+        self, migration_context, metadata=None, opts=None, autogenerate=True
+    ):
 
-        if autogenerate and \
-                migration_context is not None and migration_context.as_sql:
+        if (
+            autogenerate
+            and migration_context is not None
+            and migration_context.as_sql
+        ):
             raise util.CommandError(
                 "autogenerate can't use as_sql=True as it prevents querying "
-                "the database for schema information")
+                "the database for schema information"
+            )
 
         if opts is None:
             opts = migration_context.opts
 
-        self.metadata = metadata = opts.get('target_metadata', None) \
-            if metadata is None else metadata
+        self.metadata = metadata = (
+            opts.get("target_metadata", None) if metadata is None else metadata
+        )
 
-        if autogenerate and metadata is None and \
-                migration_context is not None and \
-                migration_context.script is not None:
+        if (
+            autogenerate
+            and metadata is None
+            and migration_context is not None
+            and migration_context.script is not None
+        ):
             raise util.CommandError(
                 "Can't proceed with --autogenerate option; environment "
                 "script %s does not provide "
-                "a MetaData object or sequence of objects to the context." % (
-                    migration_context.script.env_py_location
-                ))
+                "a MetaData object or sequence of objects to the context."
+                % (migration_context.script.env_py_location)
+            )
 
-        include_symbol = opts.get('include_symbol', None)
-        include_object = opts.get('include_object', None)
+        include_object = opts.get("include_object", None)
+        include_name = opts.get("include_name", None)
 
         object_filters = []
-        if include_symbol:
-            def include_symbol_filter(
-                    object, name, type_, reflected, compare_to):
-                if type_ == "table":
-                    return include_symbol(name, object.schema)
-                else:
-                    return True
-            object_filters.append(include_symbol_filter)
+        name_filters = []
         if include_object:
             object_filters.append(include_object)
+        if include_name:
+            name_filters.append(include_name)
 
         self._object_filters = object_filters
+        self._name_filters = name_filters
 
         self.migration_context = migration_context
         if self.migration_context is not None:
@@ -293,7 +306,7 @@ class AutogenContext(object):
 
     @util.memoized_property
     def inspector(self):
-        return Inspector.from_engine(self.connection)
+        return inspect(self.connection)
 
     @contextlib.contextmanager
     def _within_batch(self):
@@ -301,7 +314,40 @@ class AutogenContext(object):
         yield
         self._has_batch = False
 
-    def run_filters(self, object_, name, type_, reflected, compare_to):
+    def run_name_filters(self, name, type_, parent_names):
+        """Run the context's name filters and return True if the targets
+        should be part of the autogenerate operation.
+
+        This method should be run for every kind of name encountered within the
+        reflection side of an autogenerate operation, giving the environment
+        the chance to filter what names should be reflected as database
+        objects.  The filters here are produced directly via the
+        :paramref:`.EnvironmentContext.configure.include_name` parameter.
+
+        """
+
+        if "schema_name" in parent_names:
+            if type_ == "table":
+                table_name = name
+            else:
+                table_name = parent_names["table_name"]
+            schema_name = parent_names["schema_name"]
+            if schema_name:
+                parent_names["schema_qualified_table_name"] = "%s.%s" % (
+                    schema_name,
+                    table_name,
+                )
+            else:
+                parent_names["schema_qualified_table_name"] = table_name
+
+        for fn in self._name_filters:
+
+            if not fn(name, type_, parent_names):
+                return False
+        else:
+            return True
+
+    def run_object_filters(self, object_, name, type_, reflected, compare_to):
         """Run the context's object filters and return True if the targets
         should be part of the autogenerate operation.
 
@@ -309,9 +355,7 @@ class AutogenContext(object):
         an autogenerate operation, giving the environment the chance
         to filter what objects should be included in the comparison.
         The filters here are produced directly via the
-        :paramref:`.EnvironmentContext.configure.include_object`
-        and :paramref:`.EnvironmentContext.configure.include_symbol`
-        functions, if present.
+        :paramref:`.EnvironmentContext.configure.include_object` parameter.
 
         """
         for fn in self._object_filters:
@@ -320,6 +364,8 @@ class AutogenContext(object):
         else:
             return True
 
+    run_filters = run_object_filters
+
     @util.memoized_property
     def sorted_tables(self):
         """Return an aggregate of the :attr:`.MetaData.sorted_tables` collection(s).
@@ -329,8 +375,6 @@ class AutogenContext(object):
         for each individual :class:`.MetaData`  in the order of the
         sequence.  It does **not** collate the sorted tables collections.
 
-        .. versionadded:: 0.9.0
-
         """
         result = []
         for m in util.to_list(self.metadata):
@@ -348,8 +392,6 @@ class AutogenContext(object):
         Duplicate table keys are **not** supported; if two :class:`.MetaData`
         objects contain the same table key, an exception is raised.
 
-        .. versionadded:: 0.9.0
-
         """
         result = {}
         for m in util.to_list(self.metadata):
@@ -357,8 +399,8 @@ class AutogenContext(object):
             if intersect:
                 raise ValueError(
                     "Duplicate table keys across multiple "
-                    "MetaData objects: %s" %
-                    (", ".join('"%s"' % key for key in sorted(intersect)))
+                    "MetaData objects: %s"
+                    % (", ".join('"%s"' % key for key in sorted(intersect)))
                 )
 
             result.update(m.tables)
@@ -369,26 +411,29 @@ class RevisionContext(object):
     """Maintains configuration and state that's specific to a revision
     file generation operation."""
 
-    def __init__(self, config, script_directory, command_args,
-                 process_revision_directives=None):
+    def __init__(
+        self,
+        config,
+        script_directory,
+        command_args,
+        process_revision_directives=None,
+    ):
         self.config = config
         self.script_directory = script_directory
         self.command_args = command_args
         self.process_revision_directives = process_revision_directives
         self.template_args = {
-            'config': config  # Let templates use config for
-                              # e.g. multiple databases
+            "config": config  # Let templates use config for
+            # e.g. multiple databases
         }
-        self.generated_revisions = [
-            self._default_revision()
-        ]
+        self.generated_revisions = [self._default_revision()]
 
     def _to_script(self, migration_script):
         template_args = {}
         for k, v in self.template_args.items():
             template_args.setdefault(k, v)
 
-        if getattr(migration_script, '_needs_render', False):
+        if getattr(migration_script, "_needs_render", False):
             autogen_context = self._last_autogen_context
 
             # clear out existing imports if we are doing multiple
@@ -409,7 +454,8 @@ class RevisionContext(object):
             branch_labels=migration_script.branch_label,
             version_path=migration_script.version_path,
             depends_on=migration_script.depends_on,
-            **template_args)
+            **template_args
+        )
 
     def run_autogenerate(self, rev, migration_context):
         self._run_environment(rev, migration_context, True)
@@ -419,21 +465,24 @@ class RevisionContext(object):
 
     def _run_environment(self, rev, migration_context, autogenerate):
         if autogenerate:
-            if self.command_args['sql']:
+            if self.command_args["sql"]:
                 raise util.CommandError(
-                    "Using --sql with --autogenerate does not make any sense")
-            if set(self.script_directory.get_revisions(rev)) != \
-                    set(self.script_directory.get_revisions("heads")):
+                    "Using --sql with --autogenerate does not make any sense"
+                )
+            if set(self.script_directory.get_revisions(rev)) != set(
+                self.script_directory.get_revisions("heads")
+            ):
                 raise util.CommandError("Target database is not up to date.")
 
-        upgrade_token = migration_context.opts['upgrade_token']
-        downgrade_token = migration_context.opts['downgrade_token']
+        upgrade_token = migration_context.opts["upgrade_token"]
+        downgrade_token = migration_context.opts["downgrade_token"]
 
         migration_script = self.generated_revisions[-1]
-        if not getattr(migration_script, '_needs_render', False):
+        if not getattr(migration_script, "_needs_render", False):
             migration_script.upgrade_ops_list[-1].upgrade_token = upgrade_token
-            migration_script.downgrade_ops_list[-1].downgrade_token = \
-                downgrade_token
+            migration_script.downgrade_ops_list[
+                -1
+            ].downgrade_token = downgrade_token
             migration_script._needs_render = True
         else:
             migration_script._upgrade_ops.append(
@@ -443,18 +492,21 @@ class RevisionContext(object):
                 ops.DowngradeOps([], downgrade_token=downgrade_token)
             )
 
-        self._last_autogen_context = autogen_context = \
-            AutogenContext(migration_context, autogenerate=autogenerate)
+        self._last_autogen_context = autogen_context = AutogenContext(
+            migration_context, autogenerate=autogenerate
+        )
 
         if autogenerate:
             compare._populate_migration_script(
-                autogen_context, migration_script)
+                autogen_context, migration_script
+            )
 
         if self.process_revision_directives:
             self.process_revision_directives(
-                migration_context, rev, self.generated_revisions)
+                migration_context, rev, self.generated_revisions
+            )
 
-        hook = migration_context.opts['process_revision_directives']
+        hook = migration_context.opts["process_revision_directives"]
         if hook:
             hook(migration_context, rev, self.generated_revisions)
 
@@ -463,15 +515,15 @@ class RevisionContext(object):
 
     def _default_revision(self):
         op = ops.MigrationScript(
-            rev_id=self.command_args['rev_id'] or util.rev_id(),
-            message=self.command_args['message'],
+            rev_id=self.command_args["rev_id"] or util.rev_id(),
+            message=self.command_args["message"],
             upgrade_ops=ops.UpgradeOps([]),
             downgrade_ops=ops.DowngradeOps([]),
-            head=self.command_args['head'],
-            splice=self.command_args['splice'],
-            branch_label=self.command_args['branch_label'],
-            version_path=self.command_args['version_path'],
-            depends_on=self.command_args['depends_on']
+            head=self.command_args["head"],
+            splice=self.command_args["splice"],
+            branch_label=self.command_args["branch_label"],
+            version_path=self.command_args["version_path"],
+            depends_on=self.command_args["depends_on"],
         )
         return op
 
